import React from "react"
import { SafeAreaView, LogBox} from "react-native"
import { StatusBar } from "expo-status-bar"
import * as ExpoSplashScreen from 'expo-splash-screen'
import * as Updates from "expo-updates"
import { Provider, Qili }  from "./store"
import setDefaultStyle, {ColorScheme} from "./components/default-style"

LogBox.ignoreAllLogs()
ExpoSplashScreen.preventAutoHideAsync()

export default function App({children, colorScheme:scheme="black", onCrash, ...props}){
    const [style, setStyle]=React.useState({})
    const [dataReady, setDataReady]=React.useState(false)

    React.useEffect(()=>{
        const color=scheme=="light" ? "black" : "white"
        const backgroundColor=scheme=="light" ? "white" : "black"
        const active=scheme=="light" ? "black" : "white"
        const primary=scheme=="light" ? "blue" : "yellow"
        const inactive="gray"
        
        setDefaultStyle({
            Text:{color},
            MaterialIcons:{
                color:inactive,
                size:24
            },
            ActivityIndicator:{
                color,
            }
        })
        setStyle({text:color,backgroundColor,active, inactive, primary, warn:"red"})
    },[scheme])

    const content=React.useMemo(()=>{
        if(dataReady){
            const containerStyle={flex:1, backgroundColor:style.backgroundColor}
            return (
                <CrashReport onCrash={onCrash}>
                    <SafeAreaView onLayout={e=>ExpoSplashScreen.hideAsync()} style={containerStyle}>
                        <ColorScheme.Provider key={scheme} value={style}>
                            {children}
                        </ColorScheme.Provider>
                    </SafeAreaView>
                </CrashReport>
            )
        }
        return null
    },[dataReady, scheme])

    return  (
        <Provider onReady={e=>setDataReady(true)} {...props}>
            {content}
            <StatusBar style="light"/>
        </Provider>
    )
}


class CrashReport extends React.Component{
    constructor(){
        super(...arguments)
        this.state={key:Date.now()}
    }

    static getDerivedStateFromError(error){
        return {key:Date.now()}
    }

    crashReport(crash){
        this.props.onCrash?.(crash)
        const {error:{componentStack, isComponentError, message, stack}, runtimeVersion, updatedId, manifestCreatedAt}=crash
        try{
            Qili.fetch({
                query:`mutation($crash:JSON!){
                    crashReport(crash:$crash)
                }`,
                variables:{
                    crash:{componentStack, isComponentError, message, stack, runtimeVersion, updatedId, manifestCreatedAt}
                }
            })
        }catch(e){

        }finally{
            console.warn(crash.error)
        }
    }
    
    componentDidCatch(error, info){
        this.crashReport({
            error, 
            info, 
            runtimeVersion: Updates.runtimeVersion, 
            manifestCreatedAt: Updates.createdAt, 
            updatedId: Updates.updatedId
        })
    }

    render(){
        return React.cloneElement(this.props.children,{key:this.state.key})
    }
}

